# HR_1.py
# 19.03.2025
# max30102-Modul benutzen
# Der Messvorgang wird mit dem Taster 0 beendet.

mw_skal = 10 # Vorgabewert für Skalierung: 5, je nach Haut, Blutdruck in den Kapillaren und Anpressdruck des Fingers zwischen 2 und 20

from machine import SPI, Pin
import vga1_16x16 as font1
import st7789
from time import sleep, ticks_ms

taster = Pin(0, Pin.IN, Pin.PULL_UP)

# Instanziierung von display...
spi = SPI(1, baudrate=20000000, polarity=1, sck=Pin(18), mosi=Pin(19))
display = st7789.ST7789(spi, 135, 240,  reset=Pin(23, Pin.OUT), cs=Pin(5, Pin.OUT), dc=Pin(16, Pin.OUT), backlight=Pin(4, Pin.OUT), rotation=3)
# Landscape
display.init()
display.fill(0)  # loeschen; Bildschirm schwarz
display.text(font1, 'MAX-Pulsmeter', 10, 10)

from max30102 import MAX30102
# Instanziierung und setup...
max30102 = MAX30102()
max30102.setup(b'\x02') # Nur Red-LED; Default ist b'\x03' (Red & IR)

print('Puls wird im Display angezeigt:')
print('nach rechts: Zeit')
print('nach oben: Intensität des vom Finger reflekierten Lichts')
print('rote vertikale Geraden im Abstand von 1 s')
print('Programm mit Taster 1 beenden')
print()

# Zuerst wird für samp_width Samples der Mittelwert mean_value berechnet,
# Danach wird bei der laufenden Messung mit
# mean_value_new = mean_value_old + (val[samp_count+1] - val[samp_count-samp_width+1])/samp_width
# ein gleitenden Mittelwert benutzt

# Zuerst samp_FIFO füllen...
samp_width = 50 # 10
samp_FIFO = []
mean_value = 0
i = 0
while i < samp_width:
    num_samples = max30102.get_data_present()
    if num_samples > 0:
        red_led, ir_led = max30102.read_fifo()
        samp_FIFO.append(red_led)
        i += 1
        mean_value = mean_value + red_led
mean_value = mean_value / samp_width

# Parameter für die Grafik und Startwerte
x_min = 0
x_max = 239
x = x_min
y_alt = 134
t0 = ticks_ms() # für vertikale Zeitmarken

# Messen und darstellen
do_again = True # wird ggf. durch Taster T0 auf False gesetzt
while do_again:
    num_samples = max30102.get_data_present()
    if num_samples > 0:
        # value_out = samp_FIFO[samp_width-1]
        red_led, ir_led = max30102.read_fifo()
        value_in = red_led
        value_out = samp_FIFO[samp_width-1]
        for i in range(1, samp_width): 
            samp_FIFO[samp_width-i] = samp_FIFO[samp_width-i-1]
        samp_FIFO[0] = value_in
        mean_value = mean_value + (value_in - value_out) / samp_width
        # Mittelwert subtrahieren, Skalierung für Display...
        mw = int((value_in - mean_value)/mw_skal) + 50 
        display.text(font1, str(mw) + '        ', 10, 30)
        y = max(134-mw, 50)
        display.line(x,y_alt,x,y,st7789.GREEN)
        
        # Zeit-Raster...
        t1 = ticks_ms()
        if t1 >= t0 + 1000 : # 1,0 s später
            t0 = t1
            display.line(x,50,x,134,st7789.RED)            
            
        # nächste Messung vorbereiten, ggf. Display löschen...
        x += 1
        if x > x_max:
            x = x_min
            display.fill(0)  # loeschen; Bildschirm schwarz
            display.text(font1, 'MAX-Pulsmeter', 10, 10)
        do_again = taster.value() # False, wenn Taster gedrückt           

print('Ende der Messung...')        





    



