# HR_2d.py
# Messung der Herzschlagfrequenz (in bpm)
# 24.03.2025
# benutzt max30102-Modul (max30102.py)

# Vorgehensweise:
# 1. Die Graphen mit den Messwerten werden wie bei dem Programm
# HR_1.py bestimmt und auf dem Display angezeigt. Dabei werden
# diese Messwerte jetzt nun mit den zugehörigen Zeiten
# in den Listen signal_data  bzw. time_data gespeichert.
# 2. Wenn der Graph (fast) vollständig dargestellt ist, Taster 1 betätigen;
# dann werden die Signal-Maxima detektiert und die Zeitintervall zwischen
# den Maximalstellen bestimmt.
# 3. Zur Kontrolle der Genauigkeit wird auch der Standardfehler sigma am
# Monitor angegeben (https://www.scribbr.de/statistik/standardfehler/)
# 4. Der bpm-Wert wird jetzt inkl. Standardfehler im Display angezeigt.


mw_skal = 5 # 5, je nach Haut, Blutdruck in den Kapillaren und Anpressdruck des Fingers zwischen 2 und 20

from machine import SPI, Pin
import vga2_16x16 as font1 # 2 für erweiterten Zeichensatz (nötig für +/-)
import st7789
from time import sleep, ticks_ms
from math import sqrt
from sys import exit

taster = Pin(0, Pin.IN, Pin.PULL_UP)

# Instanziierung von display...
spi = SPI(1, baudrate=20000000, polarity=1, sck=Pin(18), mosi=Pin(19))
display = st7789.ST7789(spi, 135, 240,  reset=Pin(23, Pin.OUT), cs=Pin(5, Pin.OUT), dc=Pin(16, Pin.OUT), backlight=Pin(4, Pin.OUT), rotation=3)
# Landscape
display.init()
display.fill(0)  # loeschen; Bildschirm schwarz
display.text(font1, 'MAX-Pulsmeter', 10, 10)

from max30102 import MAX30102
# Instanziierung und setup...
max30102 = MAX30102()
max30102.setup(b'\x02') # Nur Red-LED; Default ist b'\x03' (Red & IR)

print('Puls wird im Display angezeigt:')
print('nach rechts: Zeit')
print('nach oben: Intensität des vom Finger reflekierten Lichts')
print('rote vertikale Geraden im Abstand von 1 s')
print('aktuelles Puls-Diagramm  mit Taster 1 auswerten lassen')
print()

# Zuerst wird für samp_width Samples der Mittelwert mean_value berechnet,
# Danach wird bei der laufenden Messung mit
# mean_value_new = mean_value_old + (val[samp_count+1] - val[samp_count-samp_width+1])/samp_width
# ein gleitenden Mittelwert benutzt

# Zuerst samp_FIFO füllen...
samp_width = 50 # 10
samp_FIFO = []
mean_value = 0
i = 0
while i < samp_width:
    num_samples = max30102.get_data_present()
    if num_samples > 0:
        red_led, ir_led = max30102.read_fifo()
        # red_led = i*10 # zum Testen
        samp_FIFO.append(red_led)
        # print(i, red_led)
        i += 1
        mean_value = mean_value + red_led
mean_value = mean_value / samp_width
# print('First Samples:', samp_FIFO)
# print('Mittelwert:', mean_value)

x_min = 0
x_max = 239
x = x_min
y_alt = 134
t0 = ticks_ms() # für vertikale Zeitmarken
do_again = True
signal_data = []
time_data = []

while do_again:
    num_samples = max30102.get_data_present()
    # print('num_samples:', num_samples)
    if num_samples > 0:
        # mean_value_new = mean_value_old + (val[samp_count+1] + val[samp_count-samp_width+1])/samp_width 
        red_led, ir_led = max30102.read_fifo()
        # red_led = 110 # zum Testen
        value_in = red_led
        # print('v_i', value_in)
        value_out = samp_FIFO[samp_width-1]
        # print('v_o', value_out)
        # FIFO-Rechts-Shift...
        for i in range(1, samp_width): # 1 bis 9
            samp_FIFO[samp_width-i] = samp_FIFO[samp_width-i-1]
            # print('i', i)             # zum Testen
        samp_FIFO[0] = value_in
        # print(samp_FIFO)
        # print(mean_value, end = ', ')
        mean_value = mean_value + (value_in - value_out) / samp_width
        # print(value_in, value_out, mean_value) # zum Testen
        # display.text(font1, str(value_in) + '        ', 10, 50) # zum Testen
        mw = int((value_in - mean_value)/mw_skal) + 50 # Skalierung für Display
        # print(line_length, mw)
        display.text(font1, str(mw) + '        ', 10, 30)
        # print(value_in)
        y = max(134-mw, 50)
        display.line(x,y_alt,x,y,st7789.GREEN)
        signal_data.append(y)

        # Zeit-Raster...
        t1 = ticks_ms()
        time_data.append(t1)
        if t1 >= t0 + 1000 : # 1,0 s später
            t0 = t1
            display.line(x,50,x,134,st7789.RED)
            
        # nächste Messung vorbereiten, ggf. Display löschen...
        x += 1
        if x > x_max:
            signal_data = []
            time_data = [t1]
            x = x_min
            display.fill(0)  # loeschen; Bildschirm schwarz
            display.text(font1, 'MAX-Pulsmeter', 10, 10)
        do_again = taster.value() # False, wenn Taster gedrückt           

print('Ende der Messung')

print('Auswertung...')

# Statt der Zeitwerte benutzen wir der Anschaulichkeit halber im Folgenden
# die entsprechende x-Werte des Diagramms

# Rohliste der Maximalstellen (x_max_list)...
w = 5 # Unschärfe bei der ersten Bestimmung der Maximalstellen, ??? ggf. = deltaT
last_x = x # letzter/größter x-Wert (entspricht letztem Zeitwert) des Diagramms
x_max_list = [] # Kandidaten für Maximalstellen (x-Werte im Diagramm)
# t_max_list = [] # Kandidaten für Maximalstellen (zugehörige t-Werte)
for x in range(w, last_x-w): 
    # zählen, wie häufig... 
    count = 0
    for x_c in range(x-w, x+w+1):
        # x_c geht von x-w bis x+w; in der Mitte x; insgesamt 2*w+1 Werte
        if signal_data[x_c] < signal_data[x]: # < oder <= ??? 
            count += 1
    if count > 2*w - 2: # fast alle y-Werte unter dem Wert an der Stelle x_c 
        x_max_list.append(x)
print('x_max_list ggf. mit mehrfachen x-Werten:', x_max_list, len(x_max_list))        
# print('t_max_list ggf. mit mehrfachen t-Werten:', t_max_list, len(t_max_list))

# jetzt dicht nebeneinander liegende Werte zu einem Mittelwert zusammenfassen...
delta_x = 10 # Unschärfe eines x-Werts bei Max-Stellen-Reduzierung
for i in range(3): # ggf. noch häufiger reduzieren
    x_max_list_reduced = []
    x = 0 # x-Wert im Diagramm (entspricht Zeit)
    while x <= len(x_max_list)-1:
        if x == len(x_max_list) - 1: # letztes Element
            x_max_list_reduced.append(x_max_list[x])
            x += 1
        elif abs(x_max_list[x] - x_max_list[x+1]) <= delta_x:
            x_max_list_reduced.append(int(round(x_max_list[x]+x_max_list[x+1])/2))                    
            x += 2
        else:
            x_max_list_reduced.append(x_max_list[x])
            x += 1
    x_max_list = x_max_list_reduced

print('x_max_list:', x_max_list)

# Zeitspannen mitteln...
diff_T = []
for i in range(len(x_max_list)-1):
    diff_T.append(time_data[x_max_list[i+1]] - time_data[x_max_list[i]])

m = 0
if diff_T == []:
    print('Kein Zeitintervall vorhanden - Abbruch')
    exit()
for i in range(len(diff_T)):
    m = m + diff_T[i]
d = m / len(diff_T) # mittlere Zeitspanne (in ms)

print('T in ms:', d)
bpm = 60/(d/1000)
print('bpm:', bpm)
bpm = round(bpm,1)
bpm_str  = str(bpm)

# Bestimmung des Standardfehlers...
print('diff_T:', diff_T) # in ms
s = 0
n = len(diff_T)
for i in range(n):
    d = 60/(diff_T[i]/1000) - bpm
    print('d:', d)
    s = s + d*d
sigma_bpm = sqrt(s/((n-1)*n)) # in s
sigma_bpm = round(sigma_bpm,1) # gerundet
print('sigma_bpm =', round(sigma_bpm), ' bpm')
sigma_bpm_str = str(sigma_bpm)

display.text(font1, bpm_str, 10, 30, st7789.RED) # 4*16 = 64 Pixel breit
display.text(font1, b'\xF1', 82, 30, st7789.RED) # 16 Pixel breit
display.text(font1, sigma_bpm_str + ' bpm', 104, 30, st7789.RED)







    



